/*
   Donkey Bolonkey
   Copyright (C) 2001 by David A. Capello

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <time.h>
#include <stdlib.h>
#include <allegro.h>
#include <allegro/internal/aintern.h>
#include "dkbk.h"
#include "joystick.h"



/* unique global variables */
DATAFILE *datafile = NULL;
COLOR_MAP *tint_map = NULL;
struct _funnel_s funnel;
struct _crusher_s crusher;



static int count;

static void inc_count()
{
  count++;
}

END_OF_STATIC_FUNCTION(inc_count);



static void play_game(BITMAP *bmp)
{
  int gameover = FALSE;

  init_level();
  init_donkeys();
  init_particles();
  init_crusher();
  init_banner();
  init_player();

  LOCK_VARIABLE(count);
  LOCK_FUNCTION(inc_count);

  count = 0;
  install_int_ex(inc_count, BPS_TO_TIMER(FRAMES_PER_SECOND));

  my_clear_keybuf();

  while (!gameover) {
    while ((count > 0) && (!gameover)) {
      poll_joystick();

      update_level();
      update_donkeys();
      update_particles();
      update_crusher();
      update_banner();

      gameover = update_player();

      count--;
    }

    clear(bmp);

    draw_level(bmp);
    draw_donkeys(bmp);
    draw_particles_first(bmp);
    draw_crusher(bmp);
    draw_banner(bmp);
    draw_particles_last(bmp);
    draw_player(bmp);

    my_flip(bmp);

    /* create new capture */
    if (key[KEY_F12])
      create_capture(bmp);

    /* pause */
    if (key[KEY_P]) {
      int old_count = count;

      textout_centre(bmp, datafile[FONT_GAME].dat, "[ PAUSE ]",
        GAME_W/2, GAME_H/3, -1);
      my_flip(bmp);

      my_clear_keybuf();
      //readkey();
      my_clear_keybuf();

      count = old_count;
    }
  }

  remove_int(inc_count);

  shutdown_level();
  shutdown_donkeys();
  shutdown_particles();
  shutdown_crusher();
  shutdown_banner();
  shutdown_player();
}



static int show_final_state;



void show_final()
{
  show_final_state = TRUE;
}



void my_clear_keybuf(void)
{
  int c;

  clear_keybuf();

  for (c=0; c<KEY_MAX; c++) {
    key[c] = 0;
    _key[c] = 0;
  }
}



void my_flip(BITMAP *bmp)
{
  if (bmp != screen) {
    vsync();
    if (GAME_W == SCREEN_W && GAME_H == SCREEN_H)
      blit(bmp, screen, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
    else
      stretch_blit(bmp, screen,
		   0, 0, GAME_W, GAME_H,
		   0, 0, SCREEN_W, SCREEN_H);
  }
}



int main(int argc, char *argv[])
{
  BITMAP *bmp;
  int first = TRUE;

  (void)argc;
  (void)argv;

  srand(time(NULL));

// gp2x screen blitter fix

// The include files needed for the below code to work
#include <sys/mman.h>
#include <allegro/platform/aintlnx.h>

unsigned long gp2x_dev;
static volatile unsigned long *gp2x_memregl;
volatile unsigned short *gp2x_memregs;

int mmsp2_blit_base;
static struct MAPPED_MEMORY mmsp2_blit;

gp2x_dev = open("/dev/mem", O_RDWR); 
gp2x_memregl = (unsigned long *)mmap(0, 0x10000, PROT_READ|PROT_WRITE, MAP_SHARED, gp2x_dev, 0xc0000000);
gp2x_memregs = (unsigned short *)gp2x_memregl;

{
     //the following is to work around bug in allegro library which causes it to lock up
     //if HW blitter hasn't done anything since GP2X was switched on.

     gp2x_memregs[0x90a >> 1] = 0xffff; // Enable all video and sound devices
     gp2x_memregs[0x904 >> 1] |= 1<<10; // Enable FASTIO for the hardware blitter

     #define MESGDSTCTRL 0x0000
     #define MESG_BSTBPP 0x60
     #define MESG_DSTBPP_16 (1<<5)
     #define MESG_DSTENB (1<<6)
     #define MESGDSTADDR 0x0004
     #define MESGSRCCTRL 0x000c
     #define MESG_INVIDEO (1<<8)
     #define MESGPATCTRL 0x0020
     #define MESGSIZE 0x002c
     #define MESG_HEIGHT 16
     #define MESGCTRL 0x0030
     #define MESG_XDIR_POS (1<<8)
     #define MESG_YDIR_POS (1<<9)
     #define MESG_FFCLR (1<<10)
     #define MESGSTATUS 0x0034
     #define MESG_BUSY (1<<0)

     #define mmsp2_blit_putl(addr,value) (*(unsigned long volatile *)(mmsp2_blit_base+(addr)) = (value))
     #define mmsp2_blit_getl(addr) (*(unsigned long volatile *)(mmsp2_blit_base+(addr)))

     mmsp2_blit.base = 0xe0020000;
     mmsp2_blit.size = 0x100;
     mmsp2_blit.perms = PROT_READ|PROT_WRITE;
     mmsp2_blit.data = mmap(0, 0x100, PROT_READ|PROT_WRITE, MAP_SHARED, gp2x_dev, 0xe0020000);

     if (mmsp2_blit.data == MAP_FAILED)
     exit(EXIT_FAILURE);

     mmsp2_blit_base = (int)mmsp2_blit.data;

     mmsp2_blit_putl (MESGDSTCTRL, MESG_DSTBPP_16 | MESG_DSTENB);
     mmsp2_blit_putl (MESGDSTADDR, 0x3101000);
     mmsp2_blit_putl (MESGSRCCTRL, MESG_INVIDEO);
     mmsp2_blit_putl (MESGPATCTRL, 0);
     mmsp2_blit_putl (MESGSIZE, (1 << MESG_HEIGHT) | 8);
     mmsp2_blit_putl (MESGCTRL, (MESG_XDIR_POS) | (MESG_YDIR_POS) | 0xaa | MESG_FFCLR);
     mmsp2_blit_putl (MESGSTATUS, 1);

     while (mmsp2_blit_getl(MESGSTATUS) & MESG_BUSY);

     munmap((void *)&mmsp2_blit_base, 0x100);
}

  allegro_init();

  set_color_depth(8);
  if (set_gfx_mode(GFX_AUTODETECT_WINDOWED, 320, 240, 0, 0) < 0) {
    if (set_gfx_mode(GFX_AUTODETECT, 320, 240, 0, 0) < 0) {
      allegro_message("Error setting graphics mode\n");
      return 1;
    }
  }

  bmp = create_bitmap(GAME_W, GAME_H);
  clear(bmp);
  text_mode(-1);
  textout(bmp, font, "Loading...", 0, 0, -1);
  my_flip(bmp);

  install_timer();
  install_keyboard();
  install_joystick(1); // gp2x joystick
  install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, NULL);

  init_graphics();
  init_hiscore();

  draw_warning(bmp);

  while (title_screen(bmp)) {
    if (first) {
      draw_controls(bmp);
      first = FALSE;
    }

    show_final_state = FALSE;
    play_game(bmp);
    if (show_final_state)
      draw_credits(screen);

    active_hiscore();
  }

  destroy_bitmap(bmp);
  shutdown_graphics();
  shutdown_hiscore();
  return 0;
}

END_OF_MAIN();


